# MyDialog.py
#
# Original version tkSimpleDialog.py - Copyright (c) 1997 by Fredrik Lundh
#
# fredrik@pythonware.com
# http://www.pythonware.com
#

# --------------------------------------------------------------------
# dialog base class

from Tkinter import *
import os

class MyDialog(Toplevel):

    def __init__(self, parent, title = None, fixed = 0):

        Toplevel.__init__(self, parent)
        self.transient(parent)

        if title:
            self.title(title)

        self.parent = parent

        self.result = None

        self.frame1 = Frame(self)
        self.frame2 = Frame(self)

        body = self.frame1 #Frame(self.frame)
        self.initial_focus = self.body(body)
        if fixed:
            body.pack(side=TOP, expand = 0, padx=5, pady=5)
        else:
            body.pack(side=TOP, expand = 0, padx = 5, pady = 5)

        if not self.initial_focus:
            self.initial_focus = self

        self.frame1.pack(side=TOP,expand=0)
        self.buttonbox()
        self.frame2.pack(side=BOTTOM)


        self.protocol("WM_DELETE_WINDOW", self.cancel)

        self.geometry("+%d+%d" % (parent.winfo_rootx()+20,
                                  parent.winfo_rooty()+20))

        self.update()

        self.resizable(1, 1)
        if fixed:
            self.resizable(0, 0)

        self.grab_set()

        self.initial_focus.focus_set()

        self.wait_window(self)

    #
    # construction hooks

    def body(self, master):
        # create dialog body.  return widget that should have
        # initial focus.  this method should be overridden

        pass

    def buttonbox(self):
        # add standard button box. override if you don't want the
        # standard buttons

        box = Frame(self.frame2)

        w = Button(box, text="OK", width=10, command=self.ok)
        w.pack(side=LEFT, padx=5, pady=5)
        w = Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=LEFT, padx=5, pady=5)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        box.pack(side=BOTTOM, fill=BOTH, expand=0)
	
    #
    # standard button semantics

    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()

        self.apply()

        self.cancel()

    def cancel(self, event=None):

        # put focus back to the parent window
        self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):

        return 1 # override

    def apply(self):

        pass # override
